"use strict";

/* exported logger */

var Panel = require('../Panel');
var PanelControlElementType = require('../PanelControlElementType');

var GuiConnector = require('../../GuiConnector');
var logger = require('../../logger');
var Functions = require('../../Functions');
var NetworkError = require('../../NetworkError');

var HttpStatus = require('http-status-codes');

function OverlayPanel(params) {
  params.panelName = "overlays";
  params.scrollable = true;
  params.helpTip = "<p>Overlays tab allows to display or generate custom coloring of elements and interactions in the map.</p>"
    + "<p>General overlays are overlays accessible for every user viewing the content.</p>"
    + "<p>Custom overlays are user-provided overlays, this menu becomes available upon login (see below).</p>";
  Panel.call(this, params);

  var self = this;

  self._createOverlayPanelGui();

  var addButton = this.getControlElement(PanelControlElementType.OVERLAY_ADD_OVERLAY_BUTTON);

  addButton.onclick = function () {
    self.openAddOverlayDialog();
  };

  var titleElement = this.getControlElement(PanelControlElementType.OVERLAY_CUSTOM_OVERLAY_TITLE);
  self.setCustomOverlaysMessage(titleElement.innerHTML);

}

OverlayPanel.prototype = Object.create(Panel.prototype);
OverlayPanel.prototype.constructor = OverlayPanel;

OverlayPanel.prototype._createOverlayPanelGui = function () {
  var generalOverlaysDiv = Functions.createElement({
    type: "div",
    name: "generalOverlays",
    className: "searchPanel"
  });
  this.getElement().appendChild(generalOverlaysDiv);
  this.setControlElement(PanelControlElementType.OVERLAY_GENERAL_OVERLAY_DIV, generalOverlaysDiv);

  var generalOverlaysTitle = Functions.createElement({
    type: "h5",
    content: "GENERAL OVERLAYS:"
  });
  generalOverlaysDiv.appendChild(generalOverlaysTitle);

  var generalOverlaysTableDiv = Functions.createElement({
    type: "table",
    name: "generalOverlaysTab",
    className: "table table-bordered",
    style: "width:100%"
  });
  generalOverlaysDiv.appendChild(generalOverlaysTableDiv);
  this.setControlElement(PanelControlElementType.OVERLAY_GENERAL_OVERLAY_TABLE, generalOverlaysTableDiv);

  var customOverlaysDiv = Functions.createElement({
    type: "div",
    name: "customOverlays",
    className: "searchPanel"
  });
  this.getElement().appendChild(customOverlaysDiv);
  this.setControlElement(PanelControlElementType.OVERLAY_CUSTOM_OVERLAY_DIV, customOverlaysDiv);

  var customOverlaysTitle = Functions.createElement({
    type: "h5",
    name: "customOverlaysTitle",
    content: "USER-PROVIDED OVERLAYS:"
  });
  customOverlaysDiv.appendChild(customOverlaysTitle);
  this.setControlElement(PanelControlElementType.OVERLAY_CUSTOM_OVERLAY_TITLE, customOverlaysTitle);

  var customOverlaysTableDiv = Functions.createElement({
    type: "table",
    name: "customOverlaysTab",
    className: "table table-bordered",
    style: "width:100%"
  });
  customOverlaysDiv.appendChild(customOverlaysTableDiv);
  this.setControlElement(PanelControlElementType.OVERLAY_CUSTOM_OVERLAY_TABLE, customOverlaysTableDiv);

  var centerTag = Functions.createElement({
    type: "center"
  });
  customOverlaysDiv.appendChild(centerTag);

  var addOverlayButton = Functions.createElement({
    type: "button",
    name: "addOverlay",
    content: "Add overlay"
  });
  centerTag.appendChild(addOverlayButton);
  this.setControlElement(PanelControlElementType.OVERLAY_ADD_OVERLAY_BUTTON, addOverlayButton);
};

OverlayPanel.prototype.clear = function () {
  var table = this.getControlElement(PanelControlElementType.OVERLAY_GENERAL_OVERLAY_TABLE);
  while (table.firstChild) {
    table.removeChild(table.firstChild);
  }

  table = this.getControlElement(PanelControlElementType.OVERLAY_CUSTOM_OVERLAY_TABLE);
  while (table.firstChild) {
    table.removeChild(table.firstChild);
  }

};

OverlayPanel.prototype.createTableHeader = function (edit) {
  var result = document.createElement("thead");

  var row = document.createElement("tr");

  var nameTd = document.createElement("th");
  nameTd.innerHTML = "Name";
  row.appendChild(nameTd);

  var viewTd = document.createElement("th");
  viewTd.innerHTML = "View";
  row.appendChild(viewTd);

  var dataTd = document.createElement("th");
  dataTd.innerHTML = "Data";
  row.appendChild(dataTd);

  if (edit) {
    var editTd = document.createElement("th");
    editTd.innerHTML = "Edit";
    row.appendChild(editTd);
  }

  result.appendChild(row);
  return result;
};

OverlayPanel.prototype.createOverlayRow = function (overlay, checked, disabled) {
  var self = this;
  var guiUtils = self.getGuiUtils();
  var result = document.createElement("tr");

  if (checked) {
    result.className = "active";
  }

  var nameTd = document.createElement("td");
  nameTd.innerHTML = overlay.getName();
  result.appendChild(nameTd);

  var viewTd = document.createElement("td");
  if (overlay.getInputDataAvailable()) {
    var checkbox = document.createElement("input");
    checkbox.type = "checkbox";
    checkbox.checked = checked;
    checkbox.onclick = function () {
      var thisCheckbox = this;
      var toggleOverlayPromise;
      if (thisCheckbox.checked) {
        toggleOverlayPromise = self.getMap().openDataOverlay(overlay.getId());
      } else {
        toggleOverlayPromise = self.getMap().removeSelectedLayout(overlay.getId());
      }
      $(thisCheckbox).prop("disabled", true);
      return toggleOverlayPromise.then(null, GuiConnector.alert).finally(function () {
        $(thisCheckbox).prop("disabled", false);
      });
    };
    $(checkbox).prop("disabled", disabled);
    viewTd.appendChild(checkbox);
  } else {
    var img = guiUtils.createIcon("icons/search.png");
    var link = document.createElement("a");
    link.href = "#";
    link.onclick = function () {
      self.getMap().openDataOverlay(overlay.getId());
      $(result).addClass('active').siblings().removeClass('active');
    };
    link.appendChild(img);
    viewTd.appendChild(link);
  }
  result.appendChild(viewTd);

  var dataTd = document.createElement("td");
  if (overlay.getInputDataAvailable()) {
    var button = document.createElement("button");
    button.setAttribute("name", "download-overlay-" + overlay.getId());
    button.onclick = function () {
      return ServerConnector.getOverlaySourceDownloadUrl({
        overlayId: overlay.getId()
      }).then(function (url) {
        return self.downloadFile(url);
      }).then(null, GuiConnector.alert);
    };
    button.innerHTML = "<span class='ui-icon ui-icon-arrowthickstop-1-s'></span>";
    dataTd.appendChild(button);
  }

  result.appendChild(dataTd);

  if (overlay.getCreator() !== "" && overlay.getCreator() !== undefined) {
    var editTd = document.createElement("td");
    var editButton = document.createElement("button");
    editButton.setAttribute("name", "editButton");
    editButton.onclick = function () {
      self.openEditOverlayDialog(overlay);
    };
    editButton.innerHTML = "<span class='ui-icon ui-icon-document'></span>";
    editTd.appendChild(editButton);
    result.appendChild(editTd);
  }
  result.title = overlay.getDescription();
  return result;
};

OverlayPanel.prototype.openEditOverlayDialog = function (overlay) {
  var self = this;
  var guiUtils = self.getGuiUtils();
  var content = document.createElement("fieldset");
  var nameInput = guiUtils.createInputText(overlay.getName());
  var row = guiUtils.createTableRow([guiUtils.createLabel("Name: "), nameInput]);
  content.appendChild(row);

  var descriptionInput = guiUtils.createTextArea(overlay.getDescription());
  row = guiUtils.createTableRow([guiUtils.createLabel("Description: "), descriptionInput]);
  content.appendChild(row);

  var consentCheckbox = document.createElement("input");
  consentCheckbox.type = "checkbox";
  consentCheckbox.checked = overlay.isGoogleLicenseConsent();
  row = guiUtils.createTableRow([guiUtils.createLabel("I am aware that this map is displayed under the terms of the <a href='https://cloud.google.com/maps-platform/terms/' target='_blank'>license of Google Maps Platform</a> and I agree to these terms. " +
    "In particular, I warrant that this dataset does not contain Protected Health Information (as defined in and subject to HIPAA). "), consentCheckbox]);
  content.appendChild(row);

  var buttons = [{
    text: "SAVE",
    click: function () {
      var windowSelf = this;
      return ServerConnector.updateOverlay({
        overlayId: overlay.id,
        name: nameInput.value,
        description: descriptionInput.value,
        googleLicenseConsent: consentCheckbox.checked
      }).then(function () {
        return self.refresh();
      }).then(function () {
        $(windowSelf).dialog("close");
      }).then(null, GuiConnector.alert);
    }
  }, {
    text: "REMOVE",
    click: function () {
      self.getMap().removeSelectedLayout(overlay.getId());

      var windowSelf = this;
      return ServerConnector.removeOverlay({
        overlayId: overlay.getId()
      }).then(function () {
        return self.refresh();
      }).then(function () {
        $(windowSelf).dialog("close");
      }).then(null, GuiConnector.alert);
    }
  }, {
    text: "CANCEL",
    click: function () {
      $(this).dialog("close");
    }
  }];
  self.openDialog(content, {
    width: "600px",
    id: overlay.getId(),
    buttons: buttons,
    title: "Data overlay: " + overlay.getName(),
    className: "minerva-overlay-dialog",
  });
};

OverlayPanel.prototype.refresh = function () {
  var self = this;
  var user = null;

  var overlayTypes = [];
  var selectedOverlay = [];

  return ServerConnector.getOverlayTypes().then(function (types) {
    overlayTypes = types;
    return ServerConnector.getLoggedUser();
  }).then(function (loggedUser) {
    user = loggedUser;
    return self.getMap().getVisibleDataOverlays();
  }).then(function (visibleDataOverlays) {
    for (var j = 0; j < visibleDataOverlays.length; j++) {
      selectedOverlay[visibleDataOverlays[j].getId()] = true;
    }

    return ServerConnector.getOverlays();
  }).then(
    function (customOverlays) {

      var id = self.getMap().getGoogleMap().getMapTypeId().substring(2);
      selectedOverlay[id] = true;

      self.clear();

      var generalOverlays = [];
      var overlay;

      var overlays = self.getMap().getLayouts();
      for (var i = 0; i < overlays.length; i++) {
        overlay = overlays[i];
        if (overlay.getCreator() === undefined || overlay.getCreator() === "") {
          generalOverlays.push(overlay);
        }
      }

      var table = self.getControlElement(PanelControlElementType.OVERLAY_GENERAL_OVERLAY_TABLE);
      table.appendChild(self.createTableHeader());

      var body = document.createElement("tbody");
      table.appendChild(body);
      for (i = 0; i < generalOverlays.length; i++) {
        overlay = generalOverlays[i];
        body.appendChild(self.createOverlayRow(overlay, selectedOverlay[overlay.getId()], false));
      }

      var title = self.getControlElement(PanelControlElementType.OVERLAY_CUSTOM_OVERLAY_TITLE);
      var addButton = self.getControlElement(PanelControlElementType.OVERLAY_ADD_OVERLAY_BUTTON);
      if (user.getLogin() === "anonymous") {
        title.innerHTML = 'YOU ARE NOT LOGGED IN. PLEASE, <a href="#">LOG IN</a>'
          + 'TO UPLOAD AND VIEW CUSTOM OVERLAYS<br/><center><button>LOGIN</button></center>';
        var openLoginDialog = function () {
          return self.getParent().getLoginDialog().open();
        };
        $(title).find("a")[0].onclick = openLoginDialog;
        $(title).find("button")[0].onclick = openLoginDialog;
        addButton.style.display = "none";
      } else {
        title.innerHTML = self.getCustomOverlaysMessage();
        addButton.style.display = "block";

        table = self.getControlElement(PanelControlElementType.OVERLAY_CUSTOM_OVERLAY_TABLE);
        table.appendChild(self.createTableHeader(true));

        body = document.createElement("tbody");
        table.appendChild(body);
        self.setGoogleLicenseConsentRequired(false);
        for (i = 0; i < customOverlays.length; i++) {
          overlay = customOverlays[i];
          body.appendChild(self.createOverlayRow(overlay, selectedOverlay[overlay.getId()], !overlay.isGoogleLicenseConsent()));
          if (!overlay.isGoogleLicenseConsent()) {
            self.setGoogleLicenseConsentRequired(true);
          }
        }
      }

      self.onresize();
    });
};

OverlayPanel.prototype.setCustomOverlaysMessage = function (customOverlaysMessage) {
  this._customOverlaysMessage = customOverlaysMessage;
};

OverlayPanel.prototype.getCustomOverlaysMessage = function () {
  return this._customOverlaysMessage;
};

OverlayPanel.prototype.parseFile = function (fileContent) {
  var result = {};
  var lines = fileContent.split("\n");
  for (var i = 0; i < lines.length; i++) {
    var line = lines[i];
    if (line.startsWith("#")) {
      if (line.indexOf("=") > 0) {
        var name = line.substring(1, line.indexOf("=")).trim();
        var value = line.substring(line.indexOf("=") + 1).trim();
        if (name === "NAME") {
          result.name = value;
        } else if (name === "DESCRIPTION") {
          result.description = value;
        } else if (name === "TYPE") {
          result.type = value;
        }
      } else {
        logger.warn("Invalid overlay header line: " + line);
      }
    } else {
      break;
    }
  }
  return result;
};

OverlayPanel.prototype.openAddOverlayDialog = function () {
  var self = this;
  var guiUtils = self.getGuiUtils();
  var fileContent = null;
  var contentInput = null;
  var content = document.createElement("div");
  content.style.width = "100%";
  content.style.height = "100%";
  content.appendChild(guiUtils.createLabel("Name: "));
  var nameInput = guiUtils.createInputText();
  content.appendChild(nameInput);
  content.appendChild(guiUtils.createNewLine());

  content.appendChild(guiUtils.createLabel("Description: "));
  content.appendChild(guiUtils.createNewLine());
  var descriptionInput = guiUtils.createTextArea();
  content.appendChild(descriptionInput);
  content.appendChild(guiUtils.createNewLine());

  content.appendChild(guiUtils.createLabel("Upload file: "));
  var fileInput = guiUtils.createFileButton();
  fileInput.addEventListener("change", function () {
    fileContent = null;
    var file = fileInput.files[0];
    if (file) {
      var reader = new FileReader();
      reader.readAsText(file, "UTF-8");
      reader.onload = function (evt) {
        fileContent = evt.target.result;
        var data = self.parseFile(fileContent);
        if (data.name !== undefined) {
          nameInput.value = data.name;
        } else {
          var filename = fileInput.value;
          if (filename.indexOf(".") > 0) {
            filename = filename.substr(0, filename.indexOf("."));
          }
          if (filename.lastIndexOf("\\") >= 0) {
            filename = filename.substr(filename.lastIndexOf("\\") + 1);
          }
          nameInput.value = filename;
        }
        if (data.description !== undefined) {
          descriptionInput.value = data.description;
        }
      };
      reader.onerror = function () {
        GuiConnector.alert("Problem reading file");
      };
    }
  }, false);
  content.appendChild(fileInput);
  content.appendChild(guiUtils.createNewLine());

  content.appendChild(guiUtils.createLabel("Or provide list of elements here (one per line): "));
  content.appendChild(guiUtils.createNewLine());
  contentInput = guiUtils.createTextArea();
  content.appendChild(contentInput);
  content.appendChild(guiUtils.createNewLine());

  var buttons = [{
    text: "UPLOAD",
    click: function () {
      var dialog = this;
      if (contentInput.value !== undefined && contentInput.value !== null) {
        contentInput.value = contentInput.value.trim();
        if (contentInput.value !== "") {
          fileContent = contentInput.value;
        }
      }
      if (fileContent === null) {
        GuiConnector.alert("Neither file was selected nor data was entered");
      } else if (fileContent.length > 1024 * 256) {
        GuiConnector.alert("File to big.<br>Please reduce file size or contact administrators.");
      } else {
        var data = {
          name: nameInput.value,
          description: descriptionInput.value,
          content: fileContent,
          filename: fileInput.value
        };
        GuiConnector.showProcessing();
        return ServerConnector.addOverlay(data).then(function (overlay) {
          self.getMap().getModel().addLayout(overlay);
          return self.refresh();
        }).then(function () {
          GuiConnector.hideProcessing();
          $(dialog).dialog("close");
        }, function (error) {
          GuiConnector.hideProcessing();
          if (error instanceof NetworkError && error.statusCode === HttpStatus.BAD_REQUEST) {
            var errorMessage = JSON.parse(error.content);
            GuiConnector.alert("Problematic input: <br/>" + errorMessage.reason);
          } else {
            GuiConnector.alert(error);
          }
        });
      }
    }
  }, {
    text: "CANCEL",
    click: function () {
      $(this).dialog("close");
    }
  }];
  self.openDialog(content, {
    id: "addOverlay",
    modal: true,
    buttons: buttons,
  });
};

OverlayPanel.prototype.init = function () {
  return this.refresh();
};

OverlayPanel.prototype.setGoogleLicenseConsentRequired = function (value) {
  this.getParent().setGoogleLicenseConsentRequired(value);
};

module.exports = OverlayPanel;
